<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;


class ParametroFolhaController extends Crud {

    protected $dataSourceName_get = 'fpa_paramfolha_get';
    protected $ParametroFolhaService;
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', "P_DTMESCOMPETENC", "P_NRPARAMFOLHAM");
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\FPA\Services\ParametroFolhaService $ParametroFolhaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->parametroFolhaService = $ParametroFolhaService;
        parent::__construct($dataSourceManager);
    }
    
    public function validaSalvar(DTO\Request\DataSet $request, DTO\Response $response){
        foreach ($request->getDataSet()->getRows() as &$row) {
            $valor = $this->parametroFolhaService->validaSalvar($row['NRGRPINCIDE']);
        }
        if($valor===false){
        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Grupo de Incidência não poderá ser utilizado. Existe um ou mais itens de incidência sem o evento informado.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
    }
    }
    
    public function getParametroFolha(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function deleteParametroFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->parametroFolhaService->deleteParametroFolha($dataset);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 