<?php

namespace HCM\Geral\Logic\ImportacaoXmlEsocial;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;


class ImportacaoXmlS2399 extends ImportacaoXmlEvento {
    
    const NMEVENTO = 'S-2399';
    
    public function __construct($nrorg, $nrorgpadrao, $dtcompetencia, $cdoperador, $xml) {
        parent::__construct($nrorg, $nrorgpadrao, $dtcompetencia, $cdoperador, $xml);
    }
    
    
    public function importarXml() {
        $nrorg       = $this->nrorg;
        $nrorgpadrao = $this->nrorgpadrao;
        $competencia = $this->dtcompetencia;
        
        $mensagens = array();
        $evtTSVTermino = (array) $this->xml->evtTSVTermino;
        
        if($evtTSVTermino) {
            
            //Aquisição das tags principais
            $ideTrabSemVinculo = (array) $evtTSVTermino['ideTrabSemVinculo'];
            $infoTSVTermino    = (array) $evtTSVTermino['infoTSVTermino'];
            
            //Dados básicos do XML do eSocial
            $cpfTrab  = isset($ideTrabSemVinculo['cpfTrab'])  ? $ideTrabSemVinculo['cpfTrab']  : null;
            $nisTrab  = isset($ideTrabSemVinculo['nisTrab'])  ? $ideTrabSemVinculo['nisTrab']  : null;
            $codCateg = isset($ideTrabSemVinculo['codCateg']) ? $ideTrabSemVinculo['codCateg'] : null;
            $dtTerm   = isset($infoTSVTermino['dtTerm'])      ? $infoTSVTermino['dtTerm']      : null;

            //Conversão de dados do XML diretamente para o formato do HCM
            $dtmescompetenc = DateUtil::getDataDeString($this->alterDateFormat($dtTerm, 1), DateUtil::FORMATO_BRASILEIRO, true);
            $DTRESCISAOVINC = DateUtil::getDataDeString($this->alterDateFormat($dtTerm, 2), DateUtil::FORMATO_BRASILEIRO, true);

            //Aquisição da pessoa para localizar o vínculo, usando o nro do cpf
            $arrayPessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtmescompetenc->format('d/m/Y'), $cpfTrab);
            if(isset($arrayPessoa[0]))
                $arrayPessoa = $arrayPessoa[0];
            if(empty($arrayPessoa)) {
                return array('status'   => false,
                             'message' => "A pessoa de CPF nro $cpfTrab não foi encontrada." );
            }
            
            //Aquisição do vínculo empregatício
            $params = array('nrcategesoc' => $codCateg);
            $objVinculoEmpreg = $this->entityManager->getRepository(Repositories::GPE_VINCULOEMPREG)->findOneBy($params);
            if(empty($objVinculoEmpreg)) {
                return array('status'   => false,
                             'message' => "Não foi encontrado um tipo de vínculo empregatício para a categoria $codCateg." );
            }
            $NRVINCULOEMPREG = $objVinculoEmpreg->getNrvinculoempreg();
            
            //Aquisicação do vínculo, busca vínculos com o número mais recente
            $NRPESSOA = $arrayPessoa['NRPESSOA'];
            $params = array('nrorg'           => $nrorg,
                            'nrpessoa'        => $NRPESSOA,
                            'nrvinculoempreg' => $NRVINCULOEMPREG);
            $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy($params, array('nrvinculom' => 'DESC'));
            if(empty($objVinculom)) {
                return array('status'   => false,
                             'message' => "Não foi encontrado nenhum vínculo para a pessoa nro $NRPESSOA com vínculo empregatício nro $NRVINCULOEMPREG." );
            }
            
            //Ajustando a data de rescisão do vínculo
            try {
                //Setando os valores
                $this->entityManager->beginTransaction();
                
                $objVinculom->setDtrescisaovinc($DTRESCISAOVINC);
                
                $this->entityManager->persist($objVinculom);
                
                $this->entityManager->flush();
                $this->entityManager->commit();
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                return array('status'   => false,
                             'message' => $e->getMessage());
            }
            
            //Retorno de sucesso
            return array('status'   => true,  
                         'messages' => $mensagens);
        } else {
            return array('status'  => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtDeslig' no arquivo).");
        }
    }
    
}