<?php

namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class ParametroEstruturaRepository {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $connection;
    
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->connection    = $connection;
    }
    
    public function getEstruturasSemParametrosEstrut($nrorg){
        try {
            $sql = <<<SQL
SELECT * 
  FROM ESTRUTURAM
 WHERE ESTRUTURAM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTURA
   AND NRORG = :P_NRORG
   AND NOT EXISTS(SELECT GPE_PARAMESTRUTM.* 
                    FROM GPE_PARAMESTRUTM  
                   WHERE GPE_PARAMESTRUTM.NRESTRUTURAM = ESTRUTURAM.NRESTRUTURAM
                     AND GPE_PARAMESTRUTM.NRORG = ESTRUTURAM.NRORG)
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_NRTIPOESTRUTURA" => $this->hcmParameters->getTipoEstruturaLegalOrg()
            );
            return $this->connection->fetchAll($sql, $params);
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function incluiParametrosEstruturaMestre($nrestruturam, $nrorg, $dtinivigencia, $dtfimvigencia){
        try {
            
            $nrparamestrutm = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PARAMESTRUTM', 1, 12, true);
            $dtatual = DateUtil::getDataAtual(true);
            $dtatual = $dtatual->format("d/m/Y H:i:s");
            $cdoperinclusao = $this->hcmParameters->getCdoperador();
            
            $params = array(
                "P_NRPARAMESTRUTM" => $nrparamestrutm,
                "P_NRORG" => $nrorg,
                "P_NRESTRUTURAM" => $nrestruturam,
                "P_DTINIVIGENCIA" => $dtinivigencia,
                "P_DTFIMVIGENCIA" => $dtfimvigencia,
                "P_DTINCLUSAO" => $dtatual,
                "P_DTULTATU" => $dtatual,
                "P_NRORGINCLUSAO" => $nrorg,
                "P_CDOPERINCLUSAO" => $cdoperinclusao,
                "P_NRORGULTATU" => $nrorg,
                "P_CDOPERULTATU" => $cdoperinclusao
            );
            
            $sql = <<<SQL
INSERT INTO GPE_PARAMESTRUTM (NRPARAMESTRUTM , NRORG, NRESTRUTURAM, DTINIVIGENCIA, DTFIMVIGENCIA, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) 
     VALUES (:P_NRPARAMESTRUTM, :P_NRORG, :P_NRESTRUTURAM, :P_DTINIVIGENCIA, :P_DTFIMVIGENCIA, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU,
             :P_CDOPERULTATU)
SQL;
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $nrparamestrutm;
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function incluiParametrosEstruturaHistorico($nrparamestrutm, $nrparamfolham, $nrparamferiasm, $nrorg, $dtmescompetenc, $nrdiainiperapuracao){
        try {
            
            $nrparamestruth = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PARAMESTRUTH', 1, 12, true);
            $dtmescompetenc = !empty($dtmescompetenc) ? $dtmescompetenc : $this->hcmParameters->getDtCompetencia();
            $dtatual = DateUtil::getDataAtual(true); 
            $dtatual = $dtatual->format("d/m/Y H:i:s");
            $cdoperinclusao = $this->hcmParameters->getCdoperador();
            $existeParametroCadastrado = $this->getParametroEstrutura($nrparamestrutm, $nrorg, $dtmescompetenc);
            
            if(!empty($existeParametroCadastrado)){
                
                $existeParametroCadastrado = $existeParametroCadastrado[0];
                
                $params = array(
                    "P_NRPARAMESTRUTH" => $existeParametroCadastrado['NRPARAMESTRUTH'],
                    "P_NRORG" => $nrorg,
                    "P_NRPARAMESTRUTM" => $nrparamestrutm,
                    "P_DTMESCOMPETENC" => $dtmescompetenc,
                    "P_DTULTATU" => $dtatual,
                    "P_NRORGULTATU" => $nrorg,
                    "P_CDOPERULTATU" => $cdoperinclusao,
                    "P_NRPARAMFERIASM" => $nrparamferiasm,
                );
                
                $sql = <<<SQL
UPDATE GPE_PARAMESTRUTH PEH 
   SET PEH.NRPARAMFERIASM = :P_NRPARAMFERIASM, 
       PEH.DTULTATU       =  :P_DTULTATU, 
       PEH.NRORGULTATU    =  :P_NRORGULTATU, 
       PEH.CDOPERULTATU   =  :P_CDOPERULTATU  
    
 WHERE PEH.NRPARAMESTRUTH          =  :P_NRPARAMESTRUTH 
       AND PEH.NRORG               =  :P_NRORG 
       AND PEH.NRPARAMESTRUTM      =  :P_NRPARAMESTRUTM 
       AND PEH.DTMESCOMPETENC      =  :P_DTMESCOMPETENC 
SQL;

            } else {
                $params = array(
                    "P_NRPARAMESTRUTH" => $nrparamestruth,
                    "P_NRORG" => $nrorg,
                    "P_NRPARAMESTRUTM" => $nrparamestrutm,
                    "P_DTMESCOMPETENC" => $dtmescompetenc,
                    "P_NRPARAMFOLHAM" => $nrparamfolham,
                    "P_DTINCLUSAO" => $dtatual,
                    "P_DTULTATU" => $dtatual,
                    "P_NRORGINCLUSAO" => $nrorg,
                    "P_CDOPERINCLUSAO" => $cdoperinclusao,
                    "P_NRORGULTATU" => $nrorg,
                    "P_CDOPERULTATU" => $cdoperinclusao,
                    "P_NRPARAMFERIASM" => $nrparamferiasm,
                    "P_NRDIAINIPERAPURACAO" => $nrdiainiperapuracao  
                );
                
                $sql = <<<SQL
INSERT INTO GPE_PARAMESTRUTH (NRPARAMESTRUTH, NRORG, NRPARAMESTRUTM, DTMESCOMPETENC, NRPARAMFOLHAM, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRPARAMFERIASM, NRDIAINIPERAPURACAO) 
 VALUES (:P_NRPARAMESTRUTH, :P_NRORG, :P_NRPARAMESTRUTM, :P_DTMESCOMPETENC, :P_NRPARAMFOLHAM, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU,
         :P_CDOPERULTATU, :P_NRPARAMFERIASM, :P_NRDIAINIPERAPURACAO)
SQL;
            }
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getUltimoHistorico($nrparamestrum, $nrorg, $dtmescompetenc){
        try {
            $params = array(
                "P_NRPARAMESTRUTM" => $nrparamestrum,
                "P_DTMESCOMPETENC" =>$dtmescompetenc,
                "P_NRORG" => $nrorg
            );
            
            $sql = <<<SQL
SELECT * FROM GPE_PARAMESTRUTH PEH
    WHERE PEH.NRPARAMESTRUTM = :P_NRPARAMESTRUTM 
      AND PEH.NRORG = :P_NRORG 
      AND PEH.DTMESCOMPETENC = (
        SELECT
            MAX(DTMESCOMPETENC)
        FROM
            GPE_PARAMESTRUTH PEH1
        WHERE
            PEH1.NRPARAMESTRUTM = PEH.NRPARAMESTRUTM
            AND PEH1.NRORG = PEH.NRORG
            AND PEH1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
      )        
SQL;

            return $this->connection->fetchAll($sql, $params);  
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getHistoricoParametroEstrutura($nrparamestrum, $nrorg, $dtmescompetenc){
        try {
            $params = array(
                "P_NRPARAMESTRUTM" => $nrparamestrum,
                "P_DTMESCOMPETENC" =>$dtmescompetenc,
                "P_NRORG" => $nrorg
            );
            
            $sql = <<<SQL
SELECT * FROM GPE_PARAMESTRUTH PEH
    WHERE PEH.NRPARAMESTRUTM = :P_NRPARAMESTRUTM 
      AND PEH.NRORG = :P_NRORG 
      AND PEH.DTMESCOMPETENC = :P_DTMESCOMPETENC        
SQL;

            return $this->connection->fetchAll($sql, $params);  
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getParametroEstrutura($nrparamestrum, $nrorg, $dtmescompetenc){
        try {
            $params = array(
                "P_NRPARAMESTRUTM" => $nrparamestrum,
                "P_DTMESCOMPETENC" =>$dtmescompetenc,
                "P_NRORG" => $nrorg
            );
            
            $sql = <<<SQL
SELECT PM.*, PH.* 
    FROM GPE_PARAMESTRUTM PM
    JOIN GPE_PARAMESTRUTH PH ON PH.NRPARAMESTRUTM = PM.NRPARAMESTRUTM
                             AND PH.NRORG = PM.NRORG
                             AND PH.DTMESCOMPETENC = :P_DTMESCOMPETENC
   WHERE PM.NRPARAMESTRUTM = :P_NRPARAMESTRUTM
          AND PM.NRORG = :P_NRORG      
SQL;

            return $this->connection->fetchAll($sql, $params);  
        } catch(\Exception $e){
            throw $e;
        }
    }
    
//     public function updateParametrosEstruturaHistorico($nrparamestrutm, $nrparamfolham, $nrparamferiasm, $nrorg, $dtmescompetenc, $nrdiainiperapuracao){
//         try {
//             $params = array(
//                 "P_NRPARAMESTRUTM" => $nrparamestrutm,
//                 "$nrparamfolham" => $nrparamfolham,
//                 "$nrparamferiasm" => $nrparamferiasm,
//                 "P_NRORG" => $nrorg,
//                 "P_DTMESCOMPETENC" =>$dtmescompetenc,
//                 "$nrdiainiperapuracao" => $nrdiainiperapuracao
//             );
            
//             $sql = <<<SQL
//       )        
// SQL;

//             return $this->connection->fetchAll($sql, $params);  
//         } catch(\Exception $e){
//             throw $e;
//         }   
//     }
    
    public function getParametrosEstruturaMestre($nrparamestrum, $nrorg){
        try {
            $params = array(
                "P_NRPARAMESTRUTM" => $nrparamestrum,
                "P_NRORG" => $nrorg
            );
            
            $sql = <<<SQL
SELECT * FROM GPE_PARAMESTRUTM PEM
    WHERE PEM.NRPARAMESTRUTM = :P_NRPARAMESTRUTM 
      AND PEM.NRORG = :P_NRORG        
SQL;

            return $this->connection->fetchAll($sql, $params);  
        } catch(\Exception $e){
            throw $e;
        }
    }
    
//     public function insereNovoHistorico($ultimoHistorico, $nrparamfolham, $dtmescompetenc, $nrorg){
//         //NRPARAMESTRUTH, NRORG, NRPARAMESTRUTM, DTMESCOMPETENC, NRPARAMFOLHAM, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRPARAMFERIASM, NRDIAINIPERAPURACAO 
//         try {
            
//             $nrparamestruth = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PARAMESTRUTH', 1, 12, true);
//             $dtmescompetenc = !empty($dtmescompetenc) ? $dtmescompetenc : $this->hcmParameters->getDtCompetencia();
//             $dtatual = DateUtil::getDataAtual(true); 
//             $dtatual = $dtatual->format("d/m/Y H:i:s");
//             $cdoperinclusao = $this->hcmParameters->getCdoperador();
            
//             $params = array(
//                 "P_NRPARAMESTRUTH" => $nrparamestruth,
//                 "P_NRORG" => $ultimoHistorico["NRORG"],
//                 "P_NRPARAMESTRUTM" => $ultimoHistorico["NRPARAMESTRUTM"],
//                 "P_DTMESCOMPETENC" => $dtmescompetenc,
//                 "P_NRPARAMFOLHAM" => $nrparamfolham,
//                 "P_DTINCLUSAO" => $dtatual,
//                 "P_DTULTATU" => $dtatual,
//                 "P_NRORGINCLUSAO" => $ultimoHistorico["NRORGINCLUSAO"],
//                 "P_CDOPERINCLUSAO" => $ultimoHistorico["CDOPERINCLUSAO"],
//                 "P_NRORGULTATU" => $ultimoHistorico["NRORGULTATU"],
//                 "P_CDOPERULTATU" => $ultimoHistorico["CDOPERULTATU"],
//                 "P_NRPARAMFERIASM" => $ultimoHistorico["NRPARAMFERIASM"],
//                 "P_NRDIAINIPERAPURACAO" =>   $ultimoHistorico["NRDIAINIPERAPURACAO"]
//             );
            
//             $sql = <<<SQL
            
// SQL;
  
//         } catch(\Exception $e){
//             throw $e;
//         }
//     }
}